/*! 
    \file factory.h
    \brief philosophy stuff
*/

#ifndef _FACTORY_H
#define _FACTORY_H

class BaseObject;

#include "xmlparser/tinyxml.h"

/** 
    Creates a factory to a Loadable Class.
    this should be used at the beginning of all the Classes that should be loadable (in the cc-file)
*/
#define CREATE_FACTORY(x) tFactory<x>* global_ ## x ## Factory = new tFactory<x>(#x)


		
//! The Factory is
/**
   Very philosophic description, huh?
*/
class Factory {

 public:
  Factory (const char* name = NULL);
  ~Factory ();
  

  virtual BaseObject* fabricate( TiXmlElement* root);
  void initialize();
  void registerFactory( Factory* factory);
  void setFactoryName(const char* name);
  const char* getFactoryName() {return factoryName;};
  void setNext( Factory* factory) {next = factory;}
  Factory* getNext() {return next;}
	
 private:
  char* factoryName;
	
  Factory* next;
};

template<class T> class tFactory : public Factory
{
 public:
  tFactory(const char* name);
  virtual ~tFactory();
  
  private:
  BaseObject* fabricate( TiXmlElement* root);
};

template<class T>
tFactory<T>::tFactory(const char* name) : Factory(name)
{
  printf("fileName: %s\n", name);
}
  

template<class T>
tFactory<T>::~tFactory()
{}

template<class T>
BaseObject* tFactory<T>::fabricate( TiXmlElement* root) 
{ 
  if(!strcmp(root->Value(), getFactoryName()))
    return new T ( root);
  else if( getNext() != NULL) 
    return getNext()->fabricate( root);
  else 
    return NULL;
}

// helper function

const char* grabParameter( TiXmlElement* root, const char* name);

#endif /* _FACTORY_H */

